# pytest framework ---------------------------------------------
import requests
from rs4 import siesta
import time
import sys
import os
import xmlrpc.client 

class Target:
    def __init__ (self, endpoint):
        self.endpoint = endpoint
        self.s = requests.Session ()
        
    def api (self, point = None):
        return siesta.API (point or self.endpoint, reraise_http_error = False)
    
    def __enter__ (self):
        return self
        
    def __exit__ (self, type, value, tb):
        self._close ()
        
    def __del__ (self):
        self._close ()
            
    def _close (self):
        pass

    def resolve (self, url):
        if url.startswith ("http://") or url.startswith ("https://"):
            return url
        else:
            return self.endpoint + url 
        
    def get (self, url, *args, **karg):
        return self.s.get (self.resolve (url), *args, **karg)
        
    def post (self, url, *args, **karg):
        return self.s.post (self.resolve (url), *args, **karg)
    
    def put (self, url, *args, **karg):
        return self.s.put (self.resolve (url), *args, **karg)
    
    def patch (self, url, *args, **karg):
        return self.s.patch (self.resolve (url), *args, **karg)
    
    def delete (self, url, *args, **karg):
        return self.s.delete (self.resolve (url), *args, **karg)
    
    def head (self, url, *args, **karg):
        return self.s.head (self.resolve (url), *args, **karg)
                
    def options (self, url, *args, **karg):
        return self.s.options (self.resolve (url), *args, **karg)
    
    def rpc (self, url, proxy_class = None):
        return (proxy_class or xmlrpc.client.ServerProxy) (self.resolve (url))
    xmlrpc = rpc
    
    def jsonrpc (self, url, proxy_class = None):
        import jsonrpclib
        return (proxy_class or jsonrpclib.ServerProxy) (self.resolve (url))
    
    def grpc (self, url, proxy_class = None):
        raise NotImplementedError
    


import asyncio as asyncio_

# asyncio abbreviations --------------------------------------------
class aio:
    @classmethod
    def loop (cls, func, *args, **karg):
        loop = asyncio_.get_event_loop ()
        loop.run_until_complete (func (*args, **karg))  
    
    @classmethod
    def wait (cls, futures, timeout = None):
        return asyncio_.wait (futures, timeout = timeout)
    
    @classmethod
    def submit (cls, func, *args, **karg):
        return asyncio_.ensure_future(func (*args, **karg)) # Future
    
    @classmethod
    async def map (cls, func, iterable):
        futures = [cls.submit (func,  item) for item in iterable]
        return await asyncio_.gather (*futures)
  
    @classmethod
    async def thread (cls, func, *args, **karg):
        loop = asyncio_.get_event_loop()  
        return await loop.run_in_executor (None, func, *args, **karg) # Future
    